# Imports the Google Cloud client library
from google.cloud import storage

from typing import List


def list_buckets(project: str) -> List:
    """Lists all buckets from a project."""
    # Instantiates a client
    storage_client = storage.Client(project=project)
    buckets = storage_client.list_buckets()
    return buckets


def get_bucket(project: str, bucket: str) -> storage.Bucket:
    """Lists all buckets."""
    # Instantiates a client
    storage_client = storage.Client(project=project)
    bucket = storage_client.get_bucket(bucket)
    return bucket



from google.cloud import resourcemanager_v3
from typing import List


def display_projects(projects: List):
    # Iterate
    for project in projects:
        print(project.project_id)


def get_list_projects() -> List:
    """Lists all projects from an account."""
    resource_manager_client = resourcemanager_v3.ProjectsClient()
    projects = resource_manager_client.search_projects()
    return projects


def get_project(project_id: str) -> List:
    """Get a project."""
    resource_manager_client = resourcemanager_v3.ProjectsClient()

    project = resource_manager_client.get_project(name=f"projects/{project_id}")
    return project



import parser
import check
import scope

_buckets_publicly_readable = []
_buckets_authenticated_readable = []


def scan_projects(options):
    projects = scope.get_projects_scope(options.project)

    # Iterate on projects
    for project in projects:

        parser.display_project(project, options)

    resume(options.resume)


def resume(resume):
    if resume:
        check.list_buckets_authenticated_readable(_buckets_authenticated_readable)
        check.list_buckets_publicly_readable(_buckets_publicly_readable)


if __name__ == "__main__":
    # Execute when the module is not initialized from an import statement.
    parser.args()



from print2 import PPrint

"""
def pretty_print(
    text: str, indent: str = "\t", prefix: str = "", level: int = 0, color: str = ""
):
    if is_multiline(text):
        for line in text.splitlines():
            pretty_print(
                text=line, prefix=prefix, indent=indent, level=level, color=color
            )
        return
    if not color:
        print(f"{indent*level}{prefix}{text}")
        return
    print(colored(f"{indent*level}{prefix}{text}", color))
    return


def is_multiline(text):
    if "\n" in text:
        return True
    return False


class PrettyPrint:

    prefix = ""
    indent = "\t"
    level = 0

    def __init__(
        self, prefix: str = "", indent: str = "\t", level: int = 0, output: int = True
    ):
        self.prefix = prefix
        self.indent = indent
        self.level = level
        self.output = output
        self.level_tmp = 0

    def set_prefix(self, prefix: str) -> None:
        self.prefix = prefix

    def get_prefix(self) -> str:
        return self.prefix

    def set_indent(self, indent: str) -> None:
        self.indent = indent

    def get_indent(self) -> str:
        return self.indent

    def set_output(self, output: bool = True) -> None:
        self.output = output

    def get_output(self) -> bool:
        return self.output

    def set_level(self, level: int) -> None:
        self.level = level

    def set_tmp_level(self, level: int) -> None:
        self.level_tmp = level

    def inc_level(self) -> None:
        self.level = self.level + 1

    def get_level(self) -> int:
        return self.level

    def print(self, text, color: str = "", output: bool = None):
        if (self.output and output is not False) or output:
            if self.level_tmp:
                pretty_print(
                    text=text,
                    prefix=self.prefix,
                    indent=self.indent,
                    level=self.level_tmp,
                    color=color,
                )
                self.level_tmp = 0
                return
            pretty_print(
                text=text,
                prefix=self.prefix,
                indent=self.indent,
                level=self.level,
                color=color,
            )
        return
"""


def display_bucket(bucket, indent, level: str = 1, output: bool = True):
    pp = PPrint(indent=indent, level=level, output=output)
    pp.print(f"{bucket.name}", output=1)
    pp.inc_level()
    time_created = str(bucket.time_created)
    pp.print(f"Created = {time_created[:10]}")
    pp.print(
        f"Uniform Bucket Level Access = {bucket.iam_configuration.uniform_bucket_level_access_enabled}"
    )
    if bucket.owner is not None:
        pp.print(f"Owner = {bucket.owner}", prefix=indent, level=level)
    pp.print(f"Location = {bucket.location} ({bucket.location_type})")
    pp.print(f"Versioning = {bucket.versioning_enabled}")
    pp.print(f"Storage class = {bucket.storage_class}")


def display_policy(policy, indent, level=0, output: bool = True) -> dict:
    _bucket_public_access = False
    _bucket_authenticated_readable = False
    pp = PPrint(indent=indent, level=level, output=output)
    pp.print("policy =")
    pp.inc_level()
    pp.print(f"version = {policy.version}")
    for binding in policy.bindings:
        pp.print(f"role = {binding['role']}")
        pp.print("member(s)=")
        for member in binding["members"]:
            pp.set_tmp_level(pp.get_level() + 1)
            pp.print(f"{member}")
            if member == "allUsers":
                _bucket_public_access = True
            if member == "allAuthenticatedUsers":
                _bucket_authenticated_readable = True
    return {
        "bucket_public_access": _bucket_public_access,
        "bucket_authenticated_readable": _bucket_authenticated_readable,
    }



import argparse
from typing import List
from logger import LOGGER
import output
import main
import scope
import utils
from print2 import PPrint, pprint


def display_project(project, options):
    _bucket_access = []
    pp = PPrint(indent="\t", level=0)
    # if project state is on DELETE_REQUESTED state (2)
    if project.state == 2:
        pprint(f"{project.display_name}", indent="", level=0, color="red")
        pprint(f"id = {project.project_id}", indent="\t", level=1, color="red")
        return []

    pp.print(f"{project.display_name}", "green")
    # pp.inc_level()
    pp.print(f"id = {project.project_id}")
    buckets = scope.get_buckets_scope(project.project_id, options.bucket)
    buckets = list(buckets)
    if not buckets:
        return
    pp.print(f"buckets =")
    for bucket in buckets:
        output.display_bucket(bucket, "\t", 1, options.details)
        utils.size_by_class(
            bucket,
            "\t",
            2,
            bucket.iam_configuration.uniform_bucket_level_access_enabled,
            display_size=options.size,
            display_file_access=options.files_access,
        )
        policy = bucket.get_iam_policy(requested_policy_version=3)
        _bucket_access = output.display_policy(
            policy,
            "\t",
            2,
            options.policy,
        )

        pp.set_level(2)
        if options.policy:
            pp.inc_level()

        if (
            "bucket_public_access" in _bucket_access
            and _bucket_access["bucket_public_access"]
        ):

            pp.print(f"Bucket publicly readable", "red")
            main._buckets_publicly_readable.append(bucket)

        elif (
            "bucket_authenticated_readable" in _bucket_access
            and _bucket_access["bucket_authenticated_readable"]
        ):
            pp.print(f"Bucket readable for anyone signed", "magenta")
            main._buckets_authenticated_readable.append(bucket)

        else:
            pp.print(
                f"Bucket not publicly and authenticated readable",
                "green",
                output=not options.only_public,
            )


def args(argv: List[str] = None) -> bool:

    if argv is None:
        import sys

        argv = sys.argv[1:]

    parser = argparse.ArgumentParser(description="What the program does")

    parser.add_argument(
        "-log", "--log", default="warning", help="Provide logging level. "
    )

    parser.add_argument(
        "-a",
        "--bucket-authenticated",
        action="store_true",
        help="Check access bucket authenticated",
    )

    parser.add_argument(
        "-o",
        "--only-public",
        action="store_true",
        help="Show only publicly readable or authenticated bucket",
    )

    parser.add_argument(
        "-f",
        "--files-access",
        action="store_true",
        help="Check access for files publicly or authenticated readable",
    )

    parser.add_argument(
        "-s",
        "--size",
        action="store_true",
        help="Show total size of bucket",
    )

    parser.add_argument(
        "-r",
        "--resume",
        action="store_true",
        help="Resume publicly readable or authenticated bucket",
    )

    parser.add_argument(
        "-p",
        "--project",
        default="",
        help="Only search in this project",
    )

    parser.add_argument(
        "-b",
        "--bucket",
        default="",
        help="Only search in this bucket",
    )

    parser.add_argument(
        "-d",
        "--details",
        action="store_true",
        help="Get details about buckets",
    )

    parser.add_argument(
        "-po",
        "--policy",
        action="store_true",
        help="Display policy",
    )

    options = parser.parse_args(argv)

    levels = {
        "critical": 50,
        "error": 40,
        "warn": 30,
        "warning": 30,
        "info": 20,
        "debug": 10,
    }
    level = levels.get(options.log.lower())

    if not level:
        raise ValueError(
            f"log level given: {options.log}"
            f" -- must be one of: {' | '.join(levels.keys())}"
        )

    LOGGER.setLevel(level)

    main.scan_projects(options)



import output


def list_buckets_publicly_readable(_buckets_publicly_readable) -> None:
    if _buckets_publicly_readable == []:
        print()
        output.pretty_print(text="0 bucket publicly readable", color="green")
        return
    print()
    output.pretty_print(text="Bucket(s) publicly readable :", color="red")
    for bucket in _buckets_publicly_readable:
        output.display_bucket(bucket, "\t")


def list_buckets_authenticated_readable(_buckets_authenticated_readable) -> None:
    if _buckets_authenticated_readable == []:
        print()
        output.pretty_print(text="0 bucket readable for anyone signed", color="green")
        return
    print()
    output.pretty_print(text="Bucket(s) readable for anyone signed:", color="magenta")
    for bucket in _buckets_authenticated_readable:
        output.display_bucket(bucket, "\t")



import math
import output
from print2 import PPrint


def convert_size(size_bytes):
    if size_bytes == 0:
        return "0 B"
    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, 2)
    return "%s %s" % (s, size_name[i])


def size_by_class(
    bucket,
    indent,
    level,
    bucket_uniform_access: bool = True,
    display_size=False,
    display_file_access=False,
):
    blobs = bucket.list_blobs()
    pp = PPrint(prefix="", indent=indent, level=level)

    size_multi_regional = (
        size_regional
    ) = size_nearline = size_coldline = size_standard = 0
    count_files = 0
    _bucket_access = {}
    for blob in blobs:
        count_files = count_files + 1
        if not bucket_uniform_access and display_file_access:
            pp.print(blob.name)
            policy = blob.get_iam_policy(requested_policy_version=3)
            _bucket_access = output.display_policy(policy, "\t", level + 1)
            if (
                "bucket_public_access" in _bucket_access
                and _bucket_access["bucket_public_access"]
            ):
                pp.print(f"Publicly readable", "red")

            elif (
                "bucket_authenticated_readable" in _bucket_access
                and _bucket_access["bucket_authenticated_readable"]
            ):
                pp.print(f"Readable for anyone signed", "magenta")

            else:
                pp.print(f"Not publicly and authenticated readable", "green")
        if blob.storage_class == "MULTI_REGIONAL":
            size_multi_regional = size_multi_regional + blob.size
        if blob.storage_class == "REGIONAL":
            size_regional = size_regional + blob.size
        if blob.storage_class == "NEARLINE":
            size_nearline = size_nearline + blob.size
        if blob.storage_class == "COLDLINE":
            size_coldline = size_coldline + blob.size
        if blob.storage_class == "STANDARD":
            size_standard = size_standard + blob.size

    if display_size:
        if size_multi_regional:
            pp.print("MULTI_REGIONAL: " + str(convert_size(size_multi_regional)))
        if size_regional:
            pp.print("REGIONAL: " + str(convert_size(size_regional)))
        if size_nearline:
            pp.print("NEARLINE: " + str(convert_size(size_nearline)))
        if size_coldline:
            pp.print("COLDLINE: " + str(convert_size(size_coldline)))
        if size_standard:
            pp.print("STANDARD: " + str(convert_size(size_standard)))

        size_total = (
            size_multi_regional
            + size_regional
            + size_nearline
            + size_coldline
            + size_standard
        )
        pp.print("TOTAL= " + str(convert_size(size_total)))
    pp.print("FILES = " + str(count_files))



import project_manager
import bucket_manager


def get_projects_scope(project: str):
    if project:
        project = project_manager.get_project(project)
        return [project]

    return project_manager.get_list_projects()


def get_buckets_scope(project_id: str, bucket: str = None):
    if bucket:
        bucket = bucket_manager.get_bucket(project_id, bucket)
        return [bucket]

    return bucket_manager.list_buckets(
        project_id,
    )
